""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class ThinStripType(enum.IntEnum):
    """The type to identify strip of face elements as thin strips using shape formed by features around the strip of face elements.
    """
    ALL = 0
    """Thin strips of all shapes."""
    EXCLUDEVSHAPE = 1
    """Thin strips of all, but not v shape."""
    VSHAPE = 2
    """Thin strips of v shape only."""

class SearchBySpikeParams(CoreObject):
    """Parameters to control spike detection.
    """
    _default_params = {}

    def __initialize(
            self,
            spike_angle: float):
        self._spike_angle = spike_angle

    def __init__(
            self,
            model: CommunicationManager=None,
            spike_angle: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SearchBySpikeParams.

        Parameters
        ----------
        model: Model
            Model to create a SearchBySpikeParams object with default parameters.
        spike_angle: float, optional
            Threshold angle for spike detection.
        json_data: dict, optional
            JSON dictionary to create a SearchBySpikeParams object with provided parameters.

        Examples
        --------
        >>> search_by_spike_params = prime.SearchBySpikeParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["spikeAngle"] if "spikeAngle" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [spike_angle])
            if all_field_specified:
                self.__initialize(
                    spike_angle)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SearchBySpikeParams")
                    json_data = param_json["SearchBySpikeParams"] if "SearchBySpikeParams" in param_json else {}
                    self.__initialize(
                        spike_angle if spike_angle is not None else ( SearchBySpikeParams._default_params["spike_angle"] if "spike_angle" in SearchBySpikeParams._default_params else (json_data["spikeAngle"] if "spikeAngle" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            spike_angle: float = None):
        """Set the default values of SearchBySpikeParams.

        Parameters
        ----------
        spike_angle: float, optional
            Threshold angle for spike detection.
        """
        args = locals()
        [SearchBySpikeParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SearchBySpikeParams.

        Examples
        --------
        >>> SearchBySpikeParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SearchBySpikeParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._spike_angle is not None:
            json_data["spikeAngle"] = self._spike_angle
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "spike_angle :  %s" % (self._spike_angle)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def spike_angle(self) -> float:
        """Threshold angle for spike detection.
        """
        return self._spike_angle

    @spike_angle.setter
    def spike_angle(self, value: float):
        self._spike_angle = value

class SearchBySpikeResults(CoreObject):
    """Results structure associated with search spikes operation.
    """
    _default_params = {}

    def __initialize(
            self,
            n_found: int,
            error_code: ErrorCode):
        self._n_found = n_found
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            n_found: int = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SearchBySpikeResults.

        Parameters
        ----------
        model: Model
            Model to create a SearchBySpikeResults object with default parameters.
        n_found: int, optional
            Number of spikes detected.
        error_code: ErrorCode, optional
            Error code associated with search spikes operation.
        json_data: dict, optional
            JSON dictionary to create a SearchBySpikeResults object with provided parameters.

        Examples
        --------
        >>> search_by_spike_results = prime.SearchBySpikeResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nFound"] if "nFound" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [n_found, error_code])
            if all_field_specified:
                self.__initialize(
                    n_found,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SearchBySpikeResults")
                    json_data = param_json["SearchBySpikeResults"] if "SearchBySpikeResults" in param_json else {}
                    self.__initialize(
                        n_found if n_found is not None else ( SearchBySpikeResults._default_params["n_found"] if "n_found" in SearchBySpikeResults._default_params else (json_data["nFound"] if "nFound" in json_data else None)),
                        error_code if error_code is not None else ( SearchBySpikeResults._default_params["error_code"] if "error_code" in SearchBySpikeResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_found: int = None,
            error_code: ErrorCode = None):
        """Set the default values of SearchBySpikeResults.

        Parameters
        ----------
        n_found: int, optional
            Number of spikes detected.
        error_code: ErrorCode, optional
            Error code associated with search spikes operation.
        """
        args = locals()
        [SearchBySpikeResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SearchBySpikeResults.

        Examples
        --------
        >>> SearchBySpikeResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SearchBySpikeResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_found is not None:
            json_data["nFound"] = self._n_found
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_found :  %s\nerror_code :  %s" % (self._n_found, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_found(self) -> int:
        """Number of spikes detected.
        """
        return self._n_found

    @n_found.setter
    def n_found(self, value: int):
        self._n_found = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with search spikes operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class SearchByFoldsParams(CoreObject):
    """Parameters to control fold detection.
    """
    _default_params = {}

    def __initialize(
            self,
            critical_angle: float):
        self._critical_angle = critical_angle

    def __init__(
            self,
            model: CommunicationManager=None,
            critical_angle: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SearchByFoldsParams.

        Parameters
        ----------
        model: Model
            Model to create a SearchByFoldsParams object with default parameters.
        critical_angle: float, optional
            Threshold angle for fold detection.
        json_data: dict, optional
            JSON dictionary to create a SearchByFoldsParams object with provided parameters.

        Examples
        --------
        >>> search_by_folds_params = prime.SearchByFoldsParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["criticalAngle"] if "criticalAngle" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [critical_angle])
            if all_field_specified:
                self.__initialize(
                    critical_angle)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SearchByFoldsParams")
                    json_data = param_json["SearchByFoldsParams"] if "SearchByFoldsParams" in param_json else {}
                    self.__initialize(
                        critical_angle if critical_angle is not None else ( SearchByFoldsParams._default_params["critical_angle"] if "critical_angle" in SearchByFoldsParams._default_params else (json_data["criticalAngle"] if "criticalAngle" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            critical_angle: float = None):
        """Set the default values of SearchByFoldsParams.

        Parameters
        ----------
        critical_angle: float, optional
            Threshold angle for fold detection.
        """
        args = locals()
        [SearchByFoldsParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SearchByFoldsParams.

        Examples
        --------
        >>> SearchByFoldsParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SearchByFoldsParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._critical_angle is not None:
            json_data["criticalAngle"] = self._critical_angle
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "critical_angle :  %s" % (self._critical_angle)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def critical_angle(self) -> float:
        """Threshold angle for fold detection.
        """
        return self._critical_angle

    @critical_angle.setter
    def critical_angle(self, value: float):
        self._critical_angle = value

class SearchByFoldsResults(CoreObject):
    """Results structure associated with search folds operation.
    """
    _default_params = {}

    def __initialize(
            self,
            n_found: int,
            error_code: ErrorCode):
        self._n_found = n_found
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            n_found: int = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SearchByFoldsResults.

        Parameters
        ----------
        model: Model
            Model to create a SearchByFoldsResults object with default parameters.
        n_found: int, optional
            Number of folds identified.
        error_code: ErrorCode, optional
            Error code associated with search folds operation.
        json_data: dict, optional
            JSON dictionary to create a SearchByFoldsResults object with provided parameters.

        Examples
        --------
        >>> search_by_folds_results = prime.SearchByFoldsResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nFound"] if "nFound" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [n_found, error_code])
            if all_field_specified:
                self.__initialize(
                    n_found,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SearchByFoldsResults")
                    json_data = param_json["SearchByFoldsResults"] if "SearchByFoldsResults" in param_json else {}
                    self.__initialize(
                        n_found if n_found is not None else ( SearchByFoldsResults._default_params["n_found"] if "n_found" in SearchByFoldsResults._default_params else (json_data["nFound"] if "nFound" in json_data else None)),
                        error_code if error_code is not None else ( SearchByFoldsResults._default_params["error_code"] if "error_code" in SearchByFoldsResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_found: int = None,
            error_code: ErrorCode = None):
        """Set the default values of SearchByFoldsResults.

        Parameters
        ----------
        n_found: int, optional
            Number of folds identified.
        error_code: ErrorCode, optional
            Error code associated with search folds operation.
        """
        args = locals()
        [SearchByFoldsResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SearchByFoldsResults.

        Examples
        --------
        >>> SearchByFoldsResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SearchByFoldsResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_found is not None:
            json_data["nFound"] = self._n_found
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_found :  %s\nerror_code :  %s" % (self._n_found, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_found(self) -> int:
        """Number of folds identified.
        """
        return self._n_found

    @n_found.setter
    def n_found(self, value: int):
        self._n_found = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with search folds operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class SearchBySelfIntersectionParams(CoreObject):
    """Parameters to search by face element intersection.
    """
    _default_params = {}

    def __initialize(
            self):
        pass

    def __init__(
            self,
            model: CommunicationManager=None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SearchBySelfIntersectionParams.

        Parameters
        ----------
        model: Model
            Model to create a SearchBySelfIntersectionParams object with default parameters.
        json_data: dict, optional
            JSON dictionary to create a SearchBySelfIntersectionParams object with provided parameters.

        Examples
        --------
        >>> search_by_self_intersection_params = prime.SearchBySelfIntersectionParams(model = model)
        """
        if json_data:
            self.__initialize()
        else:
            all_field_specified = all(arg is not None for arg in [])
            if all_field_specified:
                self.__initialize()
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SearchBySelfIntersectionParams")
                    json_data = param_json["SearchBySelfIntersectionParams"] if "SearchBySelfIntersectionParams" in param_json else {}
                    self.__initialize()
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default():
        """Set the default values of SearchBySelfIntersectionParams.

        """
        args = locals()
        [SearchBySelfIntersectionParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SearchBySelfIntersectionParams.

        Examples
        --------
        >>> SearchBySelfIntersectionParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SearchBySelfIntersectionParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "" % ()
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

class SearchByIntersectionResults(CoreObject):
    """Results associated with search by face element intersection (face elements interfering with each other).
    """
    _default_params = {}

    def __initialize(
            self,
            n_found: int,
            error_code: ErrorCode):
        self._n_found = n_found
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            n_found: int = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SearchByIntersectionResults.

        Parameters
        ----------
        model: Model
            Model to create a SearchByIntersectionResults object with default parameters.
        n_found: int, optional
            Number of face elements identified by intersection(face elements interfering with each other).
        error_code: ErrorCode, optional
            Error code associated with search intersection operation.
        json_data: dict, optional
            JSON dictionary to create a SearchByIntersectionResults object with provided parameters.

        Examples
        --------
        >>> search_by_intersection_results = prime.SearchByIntersectionResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nFound"] if "nFound" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [n_found, error_code])
            if all_field_specified:
                self.__initialize(
                    n_found,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SearchByIntersectionResults")
                    json_data = param_json["SearchByIntersectionResults"] if "SearchByIntersectionResults" in param_json else {}
                    self.__initialize(
                        n_found if n_found is not None else ( SearchByIntersectionResults._default_params["n_found"] if "n_found" in SearchByIntersectionResults._default_params else (json_data["nFound"] if "nFound" in json_data else None)),
                        error_code if error_code is not None else ( SearchByIntersectionResults._default_params["error_code"] if "error_code" in SearchByIntersectionResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_found: int = None,
            error_code: ErrorCode = None):
        """Set the default values of SearchByIntersectionResults.

        Parameters
        ----------
        n_found: int, optional
            Number of face elements identified by intersection(face elements interfering with each other).
        error_code: ErrorCode, optional
            Error code associated with search intersection operation.
        """
        args = locals()
        [SearchByIntersectionResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SearchByIntersectionResults.

        Examples
        --------
        >>> SearchByIntersectionResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SearchByIntersectionResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_found is not None:
            json_data["nFound"] = self._n_found
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_found :  %s\nerror_code :  %s" % (self._n_found, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_found(self) -> int:
        """Number of face elements identified by intersection(face elements interfering with each other).
        """
        return self._n_found

    @n_found.setter
    def n_found(self, value: int):
        self._n_found = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with search intersection operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class SearchByQualityParams(CoreObject):
    """Parameters to control search by quality results.
    """
    _default_params = {}

    def __initialize(
            self,
            quality_limit: float,
            face_quality_measure: FaceQualityMeasure):
        self._quality_limit = quality_limit
        self._face_quality_measure = FaceQualityMeasure(face_quality_measure)

    def __init__(
            self,
            model: CommunicationManager=None,
            quality_limit: float = None,
            face_quality_measure: FaceQualityMeasure = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SearchByQualityParams.

        Parameters
        ----------
        model: Model
            Model to create a SearchByQualityParams object with default parameters.
        quality_limit: float, optional
            Quality limit used for search face elements.
        face_quality_measure: FaceQualityMeasure, optional
            Quality measure used for search face elements.
        json_data: dict, optional
            JSON dictionary to create a SearchByQualityParams object with provided parameters.

        Examples
        --------
        >>> search_by_quality_params = prime.SearchByQualityParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["qualityLimit"] if "qualityLimit" in json_data else None,
                FaceQualityMeasure(json_data["faceQualityMeasure"] if "faceQualityMeasure" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [quality_limit, face_quality_measure])
            if all_field_specified:
                self.__initialize(
                    quality_limit,
                    face_quality_measure)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SearchByQualityParams")
                    json_data = param_json["SearchByQualityParams"] if "SearchByQualityParams" in param_json else {}
                    self.__initialize(
                        quality_limit if quality_limit is not None else ( SearchByQualityParams._default_params["quality_limit"] if "quality_limit" in SearchByQualityParams._default_params else (json_data["qualityLimit"] if "qualityLimit" in json_data else None)),
                        face_quality_measure if face_quality_measure is not None else ( SearchByQualityParams._default_params["face_quality_measure"] if "face_quality_measure" in SearchByQualityParams._default_params else FaceQualityMeasure(json_data["faceQualityMeasure"] if "faceQualityMeasure" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            quality_limit: float = None,
            face_quality_measure: FaceQualityMeasure = None):
        """Set the default values of SearchByQualityParams.

        Parameters
        ----------
        quality_limit: float, optional
            Quality limit used for search face elements.
        face_quality_measure: FaceQualityMeasure, optional
            Quality measure used for search face elements.
        """
        args = locals()
        [SearchByQualityParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SearchByQualityParams.

        Examples
        --------
        >>> SearchByQualityParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SearchByQualityParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._quality_limit is not None:
            json_data["qualityLimit"] = self._quality_limit
        if self._face_quality_measure is not None:
            json_data["faceQualityMeasure"] = self._face_quality_measure
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "quality_limit :  %s\nface_quality_measure :  %s" % (self._quality_limit, self._face_quality_measure)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def quality_limit(self) -> float:
        """Quality limit used for search face elements.
        """
        return self._quality_limit

    @quality_limit.setter
    def quality_limit(self, value: float):
        self._quality_limit = value

    @property
    def face_quality_measure(self) -> FaceQualityMeasure:
        """Quality measure used for search face elements.
        """
        return self._face_quality_measure

    @face_quality_measure.setter
    def face_quality_measure(self, value: FaceQualityMeasure):
        self._face_quality_measure = value

class SearchByQualityResults(CoreObject):
    """Results of search by quality.
    """
    _default_params = {}

    def __initialize(
            self,
            n_found: int,
            error_code: ErrorCode,
            max_quality: float,
            min_quality: float):
        self._n_found = n_found
        self._error_code = ErrorCode(error_code)
        self._max_quality = max_quality
        self._min_quality = min_quality

    def __init__(
            self,
            model: CommunicationManager=None,
            n_found: int = None,
            error_code: ErrorCode = None,
            max_quality: float = None,
            min_quality: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SearchByQualityResults.

        Parameters
        ----------
        model: Model
            Model to create a SearchByQualityResults object with default parameters.
        n_found: int, optional
            Number of face elements found by search for given quality limit.
        error_code: ErrorCode, optional
            Error code associated with failure of operation.
        max_quality: float, optional
            Maximum quality found by search.
        min_quality: float, optional
            Minimum quality found by search.
        json_data: dict, optional
            JSON dictionary to create a SearchByQualityResults object with provided parameters.

        Examples
        --------
        >>> search_by_quality_results = prime.SearchByQualityResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nFound"] if "nFound" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["maxQuality"] if "maxQuality" in json_data else None,
                json_data["minQuality"] if "minQuality" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [n_found, error_code, max_quality, min_quality])
            if all_field_specified:
                self.__initialize(
                    n_found,
                    error_code,
                    max_quality,
                    min_quality)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SearchByQualityResults")
                    json_data = param_json["SearchByQualityResults"] if "SearchByQualityResults" in param_json else {}
                    self.__initialize(
                        n_found if n_found is not None else ( SearchByQualityResults._default_params["n_found"] if "n_found" in SearchByQualityResults._default_params else (json_data["nFound"] if "nFound" in json_data else None)),
                        error_code if error_code is not None else ( SearchByQualityResults._default_params["error_code"] if "error_code" in SearchByQualityResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        max_quality if max_quality is not None else ( SearchByQualityResults._default_params["max_quality"] if "max_quality" in SearchByQualityResults._default_params else (json_data["maxQuality"] if "maxQuality" in json_data else None)),
                        min_quality if min_quality is not None else ( SearchByQualityResults._default_params["min_quality"] if "min_quality" in SearchByQualityResults._default_params else (json_data["minQuality"] if "minQuality" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_found: int = None,
            error_code: ErrorCode = None,
            max_quality: float = None,
            min_quality: float = None):
        """Set the default values of SearchByQualityResults.

        Parameters
        ----------
        n_found: int, optional
            Number of face elements found by search for given quality limit.
        error_code: ErrorCode, optional
            Error code associated with failure of operation.
        max_quality: float, optional
            Maximum quality found by search.
        min_quality: float, optional
            Minimum quality found by search.
        """
        args = locals()
        [SearchByQualityResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SearchByQualityResults.

        Examples
        --------
        >>> SearchByQualityResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SearchByQualityResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_found is not None:
            json_data["nFound"] = self._n_found
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._max_quality is not None:
            json_data["maxQuality"] = self._max_quality
        if self._min_quality is not None:
            json_data["minQuality"] = self._min_quality
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_found :  %s\nerror_code :  %s\nmax_quality :  %s\nmin_quality :  %s" % (self._n_found, self._error_code, self._max_quality, self._min_quality)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_found(self) -> int:
        """Number of face elements found by search for given quality limit.
        """
        return self._n_found

    @n_found.setter
    def n_found(self, value: int):
        self._n_found = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with failure of operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def max_quality(self) -> float:
        """Maximum quality found by search.
        """
        return self._max_quality

    @max_quality.setter
    def max_quality(self, value: float):
        self._max_quality = value

    @property
    def min_quality(self) -> float:
        """Minimum quality found by search.
        """
        return self._min_quality

    @min_quality.setter
    def min_quality(self, value: float):
        self._min_quality = value

class SearchByThinStripParams(CoreObject):
    """Parameters to search by thin strip of face elements.
    """
    _default_params = {}

    def __initialize(
            self,
            strip_height_limit: float,
            quality_limit: float,
            face_quality_measure: FaceQualityMeasure):
        self._strip_height_limit = strip_height_limit
        self._quality_limit = quality_limit
        self._face_quality_measure = FaceQualityMeasure(face_quality_measure)

    def __init__(
            self,
            model: CommunicationManager=None,
            strip_height_limit: float = None,
            quality_limit: float = None,
            face_quality_measure: FaceQualityMeasure = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SearchByThinStripParams.

        Parameters
        ----------
        model: Model
            Model to create a SearchByThinStripParams object with default parameters.
        strip_height_limit: float, optional
            Absolute height limit to ignore strips with height more than provided limit.
        quality_limit: float, optional
            Quality limit used for search strip of face elements.
        face_quality_measure: FaceQualityMeasure, optional
            Quality measure used for search strip of face elements.
        json_data: dict, optional
            JSON dictionary to create a SearchByThinStripParams object with provided parameters.

        Examples
        --------
        >>> search_by_thin_strip_params = prime.SearchByThinStripParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["stripHeightLimit"] if "stripHeightLimit" in json_data else None,
                json_data["qualityLimit"] if "qualityLimit" in json_data else None,
                FaceQualityMeasure(json_data["faceQualityMeasure"] if "faceQualityMeasure" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [strip_height_limit, quality_limit, face_quality_measure])
            if all_field_specified:
                self.__initialize(
                    strip_height_limit,
                    quality_limit,
                    face_quality_measure)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SearchByThinStripParams")
                    json_data = param_json["SearchByThinStripParams"] if "SearchByThinStripParams" in param_json else {}
                    self.__initialize(
                        strip_height_limit if strip_height_limit is not None else ( SearchByThinStripParams._default_params["strip_height_limit"] if "strip_height_limit" in SearchByThinStripParams._default_params else (json_data["stripHeightLimit"] if "stripHeightLimit" in json_data else None)),
                        quality_limit if quality_limit is not None else ( SearchByThinStripParams._default_params["quality_limit"] if "quality_limit" in SearchByThinStripParams._default_params else (json_data["qualityLimit"] if "qualityLimit" in json_data else None)),
                        face_quality_measure if face_quality_measure is not None else ( SearchByThinStripParams._default_params["face_quality_measure"] if "face_quality_measure" in SearchByThinStripParams._default_params else FaceQualityMeasure(json_data["faceQualityMeasure"] if "faceQualityMeasure" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            strip_height_limit: float = None,
            quality_limit: float = None,
            face_quality_measure: FaceQualityMeasure = None):
        """Set the default values of SearchByThinStripParams.

        Parameters
        ----------
        strip_height_limit: float, optional
            Absolute height limit to ignore strips with height more than provided limit.
        quality_limit: float, optional
            Quality limit used for search strip of face elements.
        face_quality_measure: FaceQualityMeasure, optional
            Quality measure used for search strip of face elements.
        """
        args = locals()
        [SearchByThinStripParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SearchByThinStripParams.

        Examples
        --------
        >>> SearchByThinStripParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SearchByThinStripParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._strip_height_limit is not None:
            json_data["stripHeightLimit"] = self._strip_height_limit
        if self._quality_limit is not None:
            json_data["qualityLimit"] = self._quality_limit
        if self._face_quality_measure is not None:
            json_data["faceQualityMeasure"] = self._face_quality_measure
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "strip_height_limit :  %s\nquality_limit :  %s\nface_quality_measure :  %s" % (self._strip_height_limit, self._quality_limit, self._face_quality_measure)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def strip_height_limit(self) -> float:
        """Absolute height limit to ignore strips with height more than provided limit.
        """
        return self._strip_height_limit

    @strip_height_limit.setter
    def strip_height_limit(self, value: float):
        self._strip_height_limit = value

    @property
    def quality_limit(self) -> float:
        """Quality limit used for search strip of face elements.
        """
        return self._quality_limit

    @quality_limit.setter
    def quality_limit(self, value: float):
        self._quality_limit = value

    @property
    def face_quality_measure(self) -> FaceQualityMeasure:
        """Quality measure used for search strip of face elements.
        """
        return self._face_quality_measure

    @face_quality_measure.setter
    def face_quality_measure(self, value: FaceQualityMeasure):
        self._face_quality_measure = value

class SearchByThinStripResults(CoreObject):
    """Results associated with search by thin strip of face elements.
    """
    _default_params = {}

    def __initialize(
            self,
            n_found: int,
            error_code: ErrorCode):
        self._n_found = n_found
        self._error_code = ErrorCode(error_code)

    def __init__(
            self,
            model: CommunicationManager=None,
            n_found: int = None,
            error_code: ErrorCode = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SearchByThinStripResults.

        Parameters
        ----------
        model: Model
            Model to create a SearchByThinStripResults object with default parameters.
        n_found: int, optional
            Number of face elements identified as thin strips.
        error_code: ErrorCode, optional
            Error code associated with search thin strips operation.
        json_data: dict, optional
            JSON dictionary to create a SearchByThinStripResults object with provided parameters.

        Examples
        --------
        >>> search_by_thin_strip_results = prime.SearchByThinStripResults(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nFound"] if "nFound" in json_data else None,
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None))
        else:
            all_field_specified = all(arg is not None for arg in [n_found, error_code])
            if all_field_specified:
                self.__initialize(
                    n_found,
                    error_code)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SearchByThinStripResults")
                    json_data = param_json["SearchByThinStripResults"] if "SearchByThinStripResults" in param_json else {}
                    self.__initialize(
                        n_found if n_found is not None else ( SearchByThinStripResults._default_params["n_found"] if "n_found" in SearchByThinStripResults._default_params else (json_data["nFound"] if "nFound" in json_data else None)),
                        error_code if error_code is not None else ( SearchByThinStripResults._default_params["error_code"] if "error_code" in SearchByThinStripResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_found: int = None,
            error_code: ErrorCode = None):
        """Set the default values of SearchByThinStripResults.

        Parameters
        ----------
        n_found: int, optional
            Number of face elements identified as thin strips.
        error_code: ErrorCode, optional
            Error code associated with search thin strips operation.
        """
        args = locals()
        [SearchByThinStripResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SearchByThinStripResults.

        Examples
        --------
        >>> SearchByThinStripResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SearchByThinStripResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_found is not None:
            json_data["nFound"] = self._n_found
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_found :  %s\nerror_code :  %s" % (self._n_found, self._error_code)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_found(self) -> int:
        """Number of face elements identified as thin strips.
        """
        return self._n_found

    @n_found.setter
    def n_found(self, value: int):
        self._n_found = value

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with search thin strips operation.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

class SurfaceQualityResult(CoreObject):
    """Result of surface quality.
    """
    _default_params = {}

    def __initialize(
            self,
            face_quality_measure: FaceQualityMeasure,
            measure_name: str,
            quality_limit: float,
            n_found: int,
            max_quality: float,
            min_quality: float):
        self._face_quality_measure = FaceQualityMeasure(face_quality_measure)
        self._measure_name = measure_name
        self._quality_limit = quality_limit
        self._n_found = n_found
        self._max_quality = max_quality
        self._min_quality = min_quality

    def __init__(
            self,
            model: CommunicationManager=None,
            face_quality_measure: FaceQualityMeasure = None,
            measure_name: str = None,
            quality_limit: float = None,
            n_found: int = None,
            max_quality: float = None,
            min_quality: float = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SurfaceQualityResult.

        Parameters
        ----------
        model: Model
            Model to create a SurfaceQualityResult object with default parameters.
        face_quality_measure: FaceQualityMeasure, optional
            Type of the face quality measure.
        measure_name: str, optional
            Name of the face quality measure.
        quality_limit: float, optional
            Target quality limit used to find failures.
        n_found: int, optional
            Number of failed faces.
        max_quality: float, optional
            Maximum value of quality measure.
        min_quality: float, optional
            Minimum value of quality measure.
        json_data: dict, optional
            JSON dictionary to create a SurfaceQualityResult object with provided parameters.

        Examples
        --------
        >>> surface_quality_result = prime.SurfaceQualityResult(model = model)
        """
        if json_data:
            self.__initialize(
                FaceQualityMeasure(json_data["faceQualityMeasure"] if "faceQualityMeasure" in json_data else None),
                json_data["measureName"] if "measureName" in json_data else None,
                json_data["qualityLimit"] if "qualityLimit" in json_data else None,
                json_data["nFound"] if "nFound" in json_data else None,
                json_data["maxQuality"] if "maxQuality" in json_data else None,
                json_data["minQuality"] if "minQuality" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [face_quality_measure, measure_name, quality_limit, n_found, max_quality, min_quality])
            if all_field_specified:
                self.__initialize(
                    face_quality_measure,
                    measure_name,
                    quality_limit,
                    n_found,
                    max_quality,
                    min_quality)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SurfaceQualityResult")
                    json_data = param_json["SurfaceQualityResult"] if "SurfaceQualityResult" in param_json else {}
                    self.__initialize(
                        face_quality_measure if face_quality_measure is not None else ( SurfaceQualityResult._default_params["face_quality_measure"] if "face_quality_measure" in SurfaceQualityResult._default_params else FaceQualityMeasure(json_data["faceQualityMeasure"] if "faceQualityMeasure" in json_data else None)),
                        measure_name if measure_name is not None else ( SurfaceQualityResult._default_params["measure_name"] if "measure_name" in SurfaceQualityResult._default_params else (json_data["measureName"] if "measureName" in json_data else None)),
                        quality_limit if quality_limit is not None else ( SurfaceQualityResult._default_params["quality_limit"] if "quality_limit" in SurfaceQualityResult._default_params else (json_data["qualityLimit"] if "qualityLimit" in json_data else None)),
                        n_found if n_found is not None else ( SurfaceQualityResult._default_params["n_found"] if "n_found" in SurfaceQualityResult._default_params else (json_data["nFound"] if "nFound" in json_data else None)),
                        max_quality if max_quality is not None else ( SurfaceQualityResult._default_params["max_quality"] if "max_quality" in SurfaceQualityResult._default_params else (json_data["maxQuality"] if "maxQuality" in json_data else None)),
                        min_quality if min_quality is not None else ( SurfaceQualityResult._default_params["min_quality"] if "min_quality" in SurfaceQualityResult._default_params else (json_data["minQuality"] if "minQuality" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            face_quality_measure: FaceQualityMeasure = None,
            measure_name: str = None,
            quality_limit: float = None,
            n_found: int = None,
            max_quality: float = None,
            min_quality: float = None):
        """Set the default values of SurfaceQualityResult.

        Parameters
        ----------
        face_quality_measure: FaceQualityMeasure, optional
            Type of the face quality measure.
        measure_name: str, optional
            Name of the face quality measure.
        quality_limit: float, optional
            Target quality limit used to find failures.
        n_found: int, optional
            Number of failed faces.
        max_quality: float, optional
            Maximum value of quality measure.
        min_quality: float, optional
            Minimum value of quality measure.
        """
        args = locals()
        [SurfaceQualityResult._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SurfaceQualityResult.

        Examples
        --------
        >>> SurfaceQualityResult.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SurfaceQualityResult._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._face_quality_measure is not None:
            json_data["faceQualityMeasure"] = self._face_quality_measure
        if self._measure_name is not None:
            json_data["measureName"] = self._measure_name
        if self._quality_limit is not None:
            json_data["qualityLimit"] = self._quality_limit
        if self._n_found is not None:
            json_data["nFound"] = self._n_found
        if self._max_quality is not None:
            json_data["maxQuality"] = self._max_quality
        if self._min_quality is not None:
            json_data["minQuality"] = self._min_quality
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "face_quality_measure :  %s\nmeasure_name :  %s\nquality_limit :  %s\nn_found :  %s\nmax_quality :  %s\nmin_quality :  %s" % (self._face_quality_measure, self._measure_name, self._quality_limit, self._n_found, self._max_quality, self._min_quality)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def face_quality_measure(self) -> FaceQualityMeasure:
        """Type of the face quality measure.
        """
        return self._face_quality_measure

    @face_quality_measure.setter
    def face_quality_measure(self, value: FaceQualityMeasure):
        self._face_quality_measure = value

    @property
    def measure_name(self) -> str:
        """Name of the face quality measure.
        """
        return self._measure_name

    @measure_name.setter
    def measure_name(self, value: str):
        self._measure_name = value

    @property
    def quality_limit(self) -> float:
        """Target quality limit used to find failures.
        """
        return self._quality_limit

    @quality_limit.setter
    def quality_limit(self, value: float):
        self._quality_limit = value

    @property
    def n_found(self) -> int:
        """Number of failed faces.
        """
        return self._n_found

    @n_found.setter
    def n_found(self, value: int):
        self._n_found = value

    @property
    def max_quality(self) -> float:
        """Maximum value of quality measure.
        """
        return self._max_quality

    @max_quality.setter
    def max_quality(self, value: float):
        self._max_quality = value

    @property
    def min_quality(self) -> float:
        """Minimum value of quality measure.
        """
        return self._min_quality

    @min_quality.setter
    def min_quality(self, value: float):
        self._min_quality = value

class SurfaceQualitySummaryResults(CoreObject):
    """Results of surface quality summary.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            quality_results: List[SurfaceQualityResult],
            summary: str):
        self._error_code = ErrorCode(error_code)
        self._quality_results = quality_results
        self._summary = summary

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            quality_results: List[SurfaceQualityResult] = None,
            summary: str = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SurfaceQualitySummaryResults.

        Parameters
        ----------
        model: Model
            Model to create a SurfaceQualitySummaryResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the surface quality summary.
        quality_results: List[SurfaceQualityResult], optional
            Contains surface quality result per face quality measure specified in parameters.
        summary: str, optional
            Surface quality summary text.
        json_data: dict, optional
            JSON dictionary to create a SurfaceQualitySummaryResults object with provided parameters.

        Examples
        --------
        >>> surface_quality_summary_results = prime.SurfaceQualitySummaryResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                [SurfaceQualityResult(model = model, json_data = data) for data in json_data["qualityResults"]] if "qualityResults" in json_data else None,
                json_data["summary"] if "summary" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, quality_results, summary])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    quality_results,
                    summary)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SurfaceQualitySummaryResults")
                    json_data = param_json["SurfaceQualitySummaryResults"] if "SurfaceQualitySummaryResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( SurfaceQualitySummaryResults._default_params["error_code"] if "error_code" in SurfaceQualitySummaryResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        quality_results if quality_results is not None else ( SurfaceQualitySummaryResults._default_params["quality_results"] if "quality_results" in SurfaceQualitySummaryResults._default_params else [SurfaceQualityResult(model = model, json_data = data) for data in (json_data["qualityResults"] if "qualityResults" in json_data else None)]),
                        summary if summary is not None else ( SurfaceQualitySummaryResults._default_params["summary"] if "summary" in SurfaceQualitySummaryResults._default_params else (json_data["summary"] if "summary" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            quality_results: List[SurfaceQualityResult] = None,
            summary: str = None):
        """Set the default values of SurfaceQualitySummaryResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the surface quality summary.
        quality_results: List[SurfaceQualityResult], optional
            Contains surface quality result per face quality measure specified in parameters.
        summary: str, optional
            Surface quality summary text.
        """
        args = locals()
        [SurfaceQualitySummaryResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SurfaceQualitySummaryResults.

        Examples
        --------
        >>> SurfaceQualitySummaryResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SurfaceQualitySummaryResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._quality_results is not None:
            json_data["qualityResults"] = [data._jsonify() for data in self._quality_results]
        if self._summary is not None:
            json_data["summary"] = self._summary
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nquality_results :  %s\nsummary :  %s" % (self._error_code, '[' + ''.join('\n' + str(data) for data in self._quality_results) + ']', self._summary)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the surface quality summary.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def quality_results(self) -> List[SurfaceQualityResult]:
        """Contains surface quality result per face quality measure specified in parameters.
        """
        return self._quality_results

    @quality_results.setter
    def quality_results(self, value: List[SurfaceQualityResult]):
        self._quality_results = value

    @property
    def summary(self) -> str:
        """Surface quality summary text.
        """
        return self._summary

    @summary.setter
    def summary(self, value: str):
        self._summary = value

class SurfaceQualitySummaryParams(CoreObject):
    """Parameters to control surface quality summary results.
    """
    _default_params = {}

    def __initialize(
            self,
            face_quality_measures: List[FaceQualityMeasure],
            scope: ScopeDefinition,
            quality_limit: Iterable[float]):
        self._face_quality_measures = face_quality_measures
        self._scope = scope
        self._quality_limit = quality_limit if isinstance(quality_limit, np.ndarray) else np.array(quality_limit, dtype=np.double) if quality_limit is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            face_quality_measures: List[FaceQualityMeasure] = None,
            scope: ScopeDefinition = None,
            quality_limit: Iterable[float] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SurfaceQualitySummaryParams.

        Parameters
        ----------
        model: Model
            Model to create a SurfaceQualitySummaryParams object with default parameters.
        face_quality_measures: List[FaceQualityMeasure], optional
            List of face quality measures for surface quality diagnostics.
        scope: ScopeDefinition, optional
            Scope the face zonelets for surface quality diagnostics.
        quality_limit: Iterable[float], optional
            Quality limit per face quality measure. If the quality limit is not specified, the default quality limit is used.
        json_data: dict, optional
            JSON dictionary to create a SurfaceQualitySummaryParams object with provided parameters.

        Examples
        --------
        >>> surface_quality_summary_params = prime.SurfaceQualitySummaryParams(model = model)
        """
        if json_data:
            self.__initialize(
                [FaceQualityMeasure(data) for data in json_data["faceQualityMeasures"]] if "faceQualityMeasures" in json_data else None,
                ScopeDefinition(model = model, json_data = json_data["scope"] if "scope" in json_data else None),
                json_data["qualityLimit"] if "qualityLimit" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [face_quality_measures, scope, quality_limit])
            if all_field_specified:
                self.__initialize(
                    face_quality_measures,
                    scope,
                    quality_limit)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SurfaceQualitySummaryParams")
                    json_data = param_json["SurfaceQualitySummaryParams"] if "SurfaceQualitySummaryParams" in param_json else {}
                    self.__initialize(
                        face_quality_measures if face_quality_measures is not None else ( SurfaceQualitySummaryParams._default_params["face_quality_measures"] if "face_quality_measures" in SurfaceQualitySummaryParams._default_params else [FaceQualityMeasure(data) for data in (json_data["faceQualityMeasures"] if "faceQualityMeasures" in json_data else None)]),
                        scope if scope is not None else ( SurfaceQualitySummaryParams._default_params["scope"] if "scope" in SurfaceQualitySummaryParams._default_params else ScopeDefinition(model = model, json_data = (json_data["scope"] if "scope" in json_data else None))),
                        quality_limit if quality_limit is not None else ( SurfaceQualitySummaryParams._default_params["quality_limit"] if "quality_limit" in SurfaceQualitySummaryParams._default_params else (json_data["qualityLimit"] if "qualityLimit" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            face_quality_measures: List[FaceQualityMeasure] = None,
            scope: ScopeDefinition = None,
            quality_limit: Iterable[float] = None):
        """Set the default values of SurfaceQualitySummaryParams.

        Parameters
        ----------
        face_quality_measures: List[FaceQualityMeasure], optional
            List of face quality measures for surface quality diagnostics.
        scope: ScopeDefinition, optional
            Scope the face zonelets for surface quality diagnostics.
        quality_limit: Iterable[float], optional
            Quality limit per face quality measure. If the quality limit is not specified, the default quality limit is used.
        """
        args = locals()
        [SurfaceQualitySummaryParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SurfaceQualitySummaryParams.

        Examples
        --------
        >>> SurfaceQualitySummaryParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SurfaceQualitySummaryParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._face_quality_measures is not None:
            json_data["faceQualityMeasures"] = [data for data in self._face_quality_measures]
        if self._scope is not None:
            json_data["scope"] = self._scope._jsonify()
        if self._quality_limit is not None:
            json_data["qualityLimit"] = self._quality_limit
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "face_quality_measures :  %s\nscope :  %s\nquality_limit :  %s" % ('[' + ''.join('\n' + str(data) for data in self._face_quality_measures) + ']', '{ ' + str(self._scope) + ' }', self._quality_limit)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def face_quality_measures(self) -> List[FaceQualityMeasure]:
        """List of face quality measures for surface quality diagnostics.
        """
        return self._face_quality_measures

    @face_quality_measures.setter
    def face_quality_measures(self, value: List[FaceQualityMeasure]):
        self._face_quality_measures = value

    @property
    def scope(self) -> ScopeDefinition:
        """Scope the face zonelets for surface quality diagnostics.
        """
        return self._scope

    @scope.setter
    def scope(self, value: ScopeDefinition):
        self._scope = value

    @property
    def quality_limit(self) -> Iterable[float]:
        """Quality limit per face quality measure. If the quality limit is not specified, the default quality limit is used.
        """
        return self._quality_limit

    @quality_limit.setter
    def quality_limit(self, value: Iterable[float]):
        self._quality_limit = value

class SurfaceDiagnosticSummaryResults(CoreObject):
    """Results of surface diagnostic summary.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            n_self_intersections: int,
            n_free_edges: int,
            n_multi_edges: int,
            n_duplicate_faces: int):
        self._error_code = ErrorCode(error_code)
        self._n_self_intersections = n_self_intersections
        self._n_free_edges = n_free_edges
        self._n_multi_edges = n_multi_edges
        self._n_duplicate_faces = n_duplicate_faces

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            n_self_intersections: int = None,
            n_free_edges: int = None,
            n_multi_edges: int = None,
            n_duplicate_faces: int = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SurfaceDiagnosticSummaryResults.

        Parameters
        ----------
        model: Model
            Model to create a SurfaceDiagnosticSummaryResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the surface diagnostic summary.
        n_self_intersections: int, optional
            Number of self intersecting faces identified.
        n_free_edges: int, optional
            Number of free face edges identified.
        n_multi_edges: int, optional
            Number of multi face edges identified.
        n_duplicate_faces: int, optional
            Number of duplicate faces identified.
        json_data: dict, optional
            JSON dictionary to create a SurfaceDiagnosticSummaryResults object with provided parameters.

        Examples
        --------
        >>> surface_diagnostic_summary_results = prime.SurfaceDiagnosticSummaryResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["nSelfIntersections"] if "nSelfIntersections" in json_data else None,
                json_data["nFreeEdges"] if "nFreeEdges" in json_data else None,
                json_data["nMultiEdges"] if "nMultiEdges" in json_data else None,
                json_data["nDuplicateFaces"] if "nDuplicateFaces" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, n_self_intersections, n_free_edges, n_multi_edges, n_duplicate_faces])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    n_self_intersections,
                    n_free_edges,
                    n_multi_edges,
                    n_duplicate_faces)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SurfaceDiagnosticSummaryResults")
                    json_data = param_json["SurfaceDiagnosticSummaryResults"] if "SurfaceDiagnosticSummaryResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( SurfaceDiagnosticSummaryResults._default_params["error_code"] if "error_code" in SurfaceDiagnosticSummaryResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        n_self_intersections if n_self_intersections is not None else ( SurfaceDiagnosticSummaryResults._default_params["n_self_intersections"] if "n_self_intersections" in SurfaceDiagnosticSummaryResults._default_params else (json_data["nSelfIntersections"] if "nSelfIntersections" in json_data else None)),
                        n_free_edges if n_free_edges is not None else ( SurfaceDiagnosticSummaryResults._default_params["n_free_edges"] if "n_free_edges" in SurfaceDiagnosticSummaryResults._default_params else (json_data["nFreeEdges"] if "nFreeEdges" in json_data else None)),
                        n_multi_edges if n_multi_edges is not None else ( SurfaceDiagnosticSummaryResults._default_params["n_multi_edges"] if "n_multi_edges" in SurfaceDiagnosticSummaryResults._default_params else (json_data["nMultiEdges"] if "nMultiEdges" in json_data else None)),
                        n_duplicate_faces if n_duplicate_faces is not None else ( SurfaceDiagnosticSummaryResults._default_params["n_duplicate_faces"] if "n_duplicate_faces" in SurfaceDiagnosticSummaryResults._default_params else (json_data["nDuplicateFaces"] if "nDuplicateFaces" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            n_self_intersections: int = None,
            n_free_edges: int = None,
            n_multi_edges: int = None,
            n_duplicate_faces: int = None):
        """Set the default values of SurfaceDiagnosticSummaryResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the surface diagnostic summary.
        n_self_intersections: int, optional
            Number of self intersecting faces identified.
        n_free_edges: int, optional
            Number of free face edges identified.
        n_multi_edges: int, optional
            Number of multi face edges identified.
        n_duplicate_faces: int, optional
            Number of duplicate faces identified.
        """
        args = locals()
        [SurfaceDiagnosticSummaryResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SurfaceDiagnosticSummaryResults.

        Examples
        --------
        >>> SurfaceDiagnosticSummaryResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SurfaceDiagnosticSummaryResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._n_self_intersections is not None:
            json_data["nSelfIntersections"] = self._n_self_intersections
        if self._n_free_edges is not None:
            json_data["nFreeEdges"] = self._n_free_edges
        if self._n_multi_edges is not None:
            json_data["nMultiEdges"] = self._n_multi_edges
        if self._n_duplicate_faces is not None:
            json_data["nDuplicateFaces"] = self._n_duplicate_faces
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nn_self_intersections :  %s\nn_free_edges :  %s\nn_multi_edges :  %s\nn_duplicate_faces :  %s" % (self._error_code, self._n_self_intersections, self._n_free_edges, self._n_multi_edges, self._n_duplicate_faces)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the surface diagnostic summary.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def n_self_intersections(self) -> int:
        """Number of self intersecting faces identified.
        """
        return self._n_self_intersections

    @n_self_intersections.setter
    def n_self_intersections(self, value: int):
        self._n_self_intersections = value

    @property
    def n_free_edges(self) -> int:
        """Number of free face edges identified.
        """
        return self._n_free_edges

    @n_free_edges.setter
    def n_free_edges(self, value: int):
        self._n_free_edges = value

    @property
    def n_multi_edges(self) -> int:
        """Number of multi face edges identified.
        """
        return self._n_multi_edges

    @n_multi_edges.setter
    def n_multi_edges(self, value: int):
        self._n_multi_edges = value

    @property
    def n_duplicate_faces(self) -> int:
        """Number of duplicate faces identified.
        """
        return self._n_duplicate_faces

    @n_duplicate_faces.setter
    def n_duplicate_faces(self, value: int):
        self._n_duplicate_faces = value

class SurfaceDiagnosticSummaryParams(CoreObject):
    """Parameters to control surface diagnostics summary results.
    """
    _default_params = {}

    def __initialize(
            self,
            scope: ScopeDefinition,
            compute_self_intersections: bool,
            compute_free_edges: bool,
            compute_multi_edges: bool,
            compute_duplicate_faces: bool):
        self._scope = scope
        self._compute_self_intersections = compute_self_intersections
        self._compute_free_edges = compute_free_edges
        self._compute_multi_edges = compute_multi_edges
        self._compute_duplicate_faces = compute_duplicate_faces

    def __init__(
            self,
            model: CommunicationManager=None,
            scope: ScopeDefinition = None,
            compute_self_intersections: bool = None,
            compute_free_edges: bool = None,
            compute_multi_edges: bool = None,
            compute_duplicate_faces: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SurfaceDiagnosticSummaryParams.

        Parameters
        ----------
        model: Model
            Model to create a SurfaceDiagnosticSummaryParams object with default parameters.
        scope: ScopeDefinition, optional
            Scope the face zonelets for surface diagnostics.
        compute_self_intersections: bool, optional
            Control to identify face intersections are present or not.
        compute_free_edges: bool, optional
            Control to identify free face edges are present or not.
        compute_multi_edges: bool, optional
            Control to identify multi face edges are present or not.
        compute_duplicate_faces: bool, optional
            Control to identify duplicate faces are present or not.
        json_data: dict, optional
            JSON dictionary to create a SurfaceDiagnosticSummaryParams object with provided parameters.

        Examples
        --------
        >>> surface_diagnostic_summary_params = prime.SurfaceDiagnosticSummaryParams(model = model)
        """
        if json_data:
            self.__initialize(
                ScopeDefinition(model = model, json_data = json_data["scope"] if "scope" in json_data else None),
                json_data["computeSelfIntersections"] if "computeSelfIntersections" in json_data else None,
                json_data["computeFreeEdges"] if "computeFreeEdges" in json_data else None,
                json_data["computeMultiEdges"] if "computeMultiEdges" in json_data else None,
                json_data["computeDuplicateFaces"] if "computeDuplicateFaces" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [scope, compute_self_intersections, compute_free_edges, compute_multi_edges, compute_duplicate_faces])
            if all_field_specified:
                self.__initialize(
                    scope,
                    compute_self_intersections,
                    compute_free_edges,
                    compute_multi_edges,
                    compute_duplicate_faces)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SurfaceDiagnosticSummaryParams")
                    json_data = param_json["SurfaceDiagnosticSummaryParams"] if "SurfaceDiagnosticSummaryParams" in param_json else {}
                    self.__initialize(
                        scope if scope is not None else ( SurfaceDiagnosticSummaryParams._default_params["scope"] if "scope" in SurfaceDiagnosticSummaryParams._default_params else ScopeDefinition(model = model, json_data = (json_data["scope"] if "scope" in json_data else None))),
                        compute_self_intersections if compute_self_intersections is not None else ( SurfaceDiagnosticSummaryParams._default_params["compute_self_intersections"] if "compute_self_intersections" in SurfaceDiagnosticSummaryParams._default_params else (json_data["computeSelfIntersections"] if "computeSelfIntersections" in json_data else None)),
                        compute_free_edges if compute_free_edges is not None else ( SurfaceDiagnosticSummaryParams._default_params["compute_free_edges"] if "compute_free_edges" in SurfaceDiagnosticSummaryParams._default_params else (json_data["computeFreeEdges"] if "computeFreeEdges" in json_data else None)),
                        compute_multi_edges if compute_multi_edges is not None else ( SurfaceDiagnosticSummaryParams._default_params["compute_multi_edges"] if "compute_multi_edges" in SurfaceDiagnosticSummaryParams._default_params else (json_data["computeMultiEdges"] if "computeMultiEdges" in json_data else None)),
                        compute_duplicate_faces if compute_duplicate_faces is not None else ( SurfaceDiagnosticSummaryParams._default_params["compute_duplicate_faces"] if "compute_duplicate_faces" in SurfaceDiagnosticSummaryParams._default_params else (json_data["computeDuplicateFaces"] if "computeDuplicateFaces" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            scope: ScopeDefinition = None,
            compute_self_intersections: bool = None,
            compute_free_edges: bool = None,
            compute_multi_edges: bool = None,
            compute_duplicate_faces: bool = None):
        """Set the default values of SurfaceDiagnosticSummaryParams.

        Parameters
        ----------
        scope: ScopeDefinition, optional
            Scope the face zonelets for surface diagnostics.
        compute_self_intersections: bool, optional
            Control to identify face intersections are present or not.
        compute_free_edges: bool, optional
            Control to identify free face edges are present or not.
        compute_multi_edges: bool, optional
            Control to identify multi face edges are present or not.
        compute_duplicate_faces: bool, optional
            Control to identify duplicate faces are present or not.
        """
        args = locals()
        [SurfaceDiagnosticSummaryParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SurfaceDiagnosticSummaryParams.

        Examples
        --------
        >>> SurfaceDiagnosticSummaryParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SurfaceDiagnosticSummaryParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._scope is not None:
            json_data["scope"] = self._scope._jsonify()
        if self._compute_self_intersections is not None:
            json_data["computeSelfIntersections"] = self._compute_self_intersections
        if self._compute_free_edges is not None:
            json_data["computeFreeEdges"] = self._compute_free_edges
        if self._compute_multi_edges is not None:
            json_data["computeMultiEdges"] = self._compute_multi_edges
        if self._compute_duplicate_faces is not None:
            json_data["computeDuplicateFaces"] = self._compute_duplicate_faces
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "scope :  %s\ncompute_self_intersections :  %s\ncompute_free_edges :  %s\ncompute_multi_edges :  %s\ncompute_duplicate_faces :  %s" % ('{ ' + str(self._scope) + ' }', self._compute_self_intersections, self._compute_free_edges, self._compute_multi_edges, self._compute_duplicate_faces)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def scope(self) -> ScopeDefinition:
        """Scope the face zonelets for surface diagnostics.
        """
        return self._scope

    @scope.setter
    def scope(self, value: ScopeDefinition):
        self._scope = value

    @property
    def compute_self_intersections(self) -> bool:
        """Control to identify face intersections are present or not.
        """
        return self._compute_self_intersections

    @compute_self_intersections.setter
    def compute_self_intersections(self, value: bool):
        self._compute_self_intersections = value

    @property
    def compute_free_edges(self) -> bool:
        """Control to identify free face edges are present or not.
        """
        return self._compute_free_edges

    @compute_free_edges.setter
    def compute_free_edges(self, value: bool):
        self._compute_free_edges = value

    @property
    def compute_multi_edges(self) -> bool:
        """Control to identify multi face edges are present or not.
        """
        return self._compute_multi_edges

    @compute_multi_edges.setter
    def compute_multi_edges(self, value: bool):
        self._compute_multi_edges = value

    @property
    def compute_duplicate_faces(self) -> bool:
        """Control to identify duplicate faces are present or not.
        """
        return self._compute_duplicate_faces

    @compute_duplicate_faces.setter
    def compute_duplicate_faces(self, value: bool):
        self._compute_duplicate_faces = value

class SearchInfoByRegisterIdParams(CoreObject):
    """Parameters to retrieve information on registered faces.
    """
    _default_params = {}

    def __initialize(
            self):
        pass

    def __init__(
            self,
            model: CommunicationManager=None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SearchInfoByRegisterIdParams.

        Parameters
        ----------
        model: Model
            Model to create a SearchInfoByRegisterIdParams object with default parameters.
        json_data: dict, optional
            JSON dictionary to create a SearchInfoByRegisterIdParams object with provided parameters.

        Examples
        --------
        >>> search_info_by_register_id_params = prime.SearchInfoByRegisterIdParams(model = model)
        """
        if json_data:
            self.__initialize()
        else:
            all_field_specified = all(arg is not None for arg in [])
            if all_field_specified:
                self.__initialize()
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SearchInfoByRegisterIdParams")
                    json_data = param_json["SearchInfoByRegisterIdParams"] if "SearchInfoByRegisterIdParams" in param_json else {}
                    self.__initialize()
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default():
        """Set the default values of SearchInfoByRegisterIdParams.

        """
        args = locals()
        [SearchInfoByRegisterIdParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SearchInfoByRegisterIdParams.

        Examples
        --------
        >>> SearchInfoByRegisterIdParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SearchInfoByRegisterIdParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "" % ()
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

class SearchInfoByRegisterIdResults(CoreObject):
    """Result structure containing information on registered face elements.
    """
    _default_params = {}

    def __initialize(
            self,
            error_code: ErrorCode,
            n_found: int,
            locations_found: Iterable[float],
            face_zonelets_found: Iterable[int]):
        self._error_code = ErrorCode(error_code)
        self._n_found = n_found
        self._locations_found = locations_found if isinstance(locations_found, np.ndarray) else np.array(locations_found, dtype=np.double) if locations_found is not None else None
        self._face_zonelets_found = face_zonelets_found if isinstance(face_zonelets_found, np.ndarray) else np.array(face_zonelets_found, dtype=np.int32) if face_zonelets_found is not None else None

    def __init__(
            self,
            model: CommunicationManager=None,
            error_code: ErrorCode = None,
            n_found: int = None,
            locations_found: Iterable[float] = None,
            face_zonelets_found: Iterable[int] = None,
            json_data : dict = None,
             **kwargs):
        """Initializes the SearchInfoByRegisterIdResults.

        Parameters
        ----------
        model: Model
            Model to create a SearchInfoByRegisterIdResults object with default parameters.
        error_code: ErrorCode, optional
            Error code associated with the retreiving information based on register id.
        n_found: int, optional
            Number of registered face elements.
        locations_found: Iterable[float], optional
            Locations of each cluster of registered face elements.
        face_zonelets_found: Iterable[int], optional
            Ids of the face zonelets containing atleast one registered face element.
        json_data: dict, optional
            JSON dictionary to create a SearchInfoByRegisterIdResults object with provided parameters.

        Examples
        --------
        >>> search_info_by_register_id_results = prime.SearchInfoByRegisterIdResults(model = model)
        """
        if json_data:
            self.__initialize(
                ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None),
                json_data["nFound"] if "nFound" in json_data else None,
                json_data["locationsFound"] if "locationsFound" in json_data else None,
                json_data["faceZoneletsFound"] if "faceZoneletsFound" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [error_code, n_found, locations_found, face_zonelets_found])
            if all_field_specified:
                self.__initialize(
                    error_code,
                    n_found,
                    locations_found,
                    face_zonelets_found)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass model or specify all properties")
                else:
                    param_json = model._communicator.initialize_params(model, "SearchInfoByRegisterIdResults")
                    json_data = param_json["SearchInfoByRegisterIdResults"] if "SearchInfoByRegisterIdResults" in param_json else {}
                    self.__initialize(
                        error_code if error_code is not None else ( SearchInfoByRegisterIdResults._default_params["error_code"] if "error_code" in SearchInfoByRegisterIdResults._default_params else ErrorCode(json_data["errorCode"] if "errorCode" in json_data else None)),
                        n_found if n_found is not None else ( SearchInfoByRegisterIdResults._default_params["n_found"] if "n_found" in SearchInfoByRegisterIdResults._default_params else (json_data["nFound"] if "nFound" in json_data else None)),
                        locations_found if locations_found is not None else ( SearchInfoByRegisterIdResults._default_params["locations_found"] if "locations_found" in SearchInfoByRegisterIdResults._default_params else (json_data["locationsFound"] if "locationsFound" in json_data else None)),
                        face_zonelets_found if face_zonelets_found is not None else ( SearchInfoByRegisterIdResults._default_params["face_zonelets_found"] if "face_zonelets_found" in SearchInfoByRegisterIdResults._default_params else (json_data["faceZoneletsFound"] if "faceZoneletsFound" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            error_code: ErrorCode = None,
            n_found: int = None,
            locations_found: Iterable[float] = None,
            face_zonelets_found: Iterable[int] = None):
        """Set the default values of SearchInfoByRegisterIdResults.

        Parameters
        ----------
        error_code: ErrorCode, optional
            Error code associated with the retreiving information based on register id.
        n_found: int, optional
            Number of registered face elements.
        locations_found: Iterable[float], optional
            Locations of each cluster of registered face elements.
        face_zonelets_found: Iterable[int], optional
            Ids of the face zonelets containing atleast one registered face element.
        """
        args = locals()
        [SearchInfoByRegisterIdResults._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of SearchInfoByRegisterIdResults.

        Examples
        --------
        >>> SearchInfoByRegisterIdResults.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in SearchInfoByRegisterIdResults._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._error_code is not None:
            json_data["errorCode"] = self._error_code
        if self._n_found is not None:
            json_data["nFound"] = self._n_found
        if self._locations_found is not None:
            json_data["locationsFound"] = self._locations_found
        if self._face_zonelets_found is not None:
            json_data["faceZoneletsFound"] = self._face_zonelets_found
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "error_code :  %s\nn_found :  %s\nlocations_found :  %s\nface_zonelets_found :  %s" % (self._error_code, self._n_found, self._locations_found, self._face_zonelets_found)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def error_code(self) -> ErrorCode:
        """Error code associated with the retreiving information based on register id.
        """
        return self._error_code

    @error_code.setter
    def error_code(self, value: ErrorCode):
        self._error_code = value

    @property
    def n_found(self) -> int:
        """Number of registered face elements.
        """
        return self._n_found

    @n_found.setter
    def n_found(self, value: int):
        self._n_found = value

    @property
    def locations_found(self) -> Iterable[float]:
        """Locations of each cluster of registered face elements.
        """
        return self._locations_found

    @locations_found.setter
    def locations_found(self, value: Iterable[float]):
        self._locations_found = value

    @property
    def face_zonelets_found(self) -> Iterable[int]:
        """Ids of the face zonelets containing atleast one registered face element.
        """
        return self._face_zonelets_found

    @face_zonelets_found.setter
    def face_zonelets_found(self, value: Iterable[int]):
        self._face_zonelets_found = value



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class TopoData(CoreObject):
    """Topodata has all information about connectivity of nodes, edges, elements and more.

    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize TopoData """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def get_adjacent_topo_faces_of_topo_faces(self, topo_faces : Iterable[int]) -> Iterable[int]:
        """ Get the adjacent topofaces for the provided topoface ids.


        Parameters
        ----------
        topo_faces : Iterable[int]
            Ids of the topoface.

        Returns
        -------
        Iterable[int]
            Returns the list of adjacent topoface ids.


        Examples
        --------
        >>> topo_faces_of_topo_faces = topo_data.get_adjacent_topo_faces_of_topo_faces(topo_faces)

        """
        args = {"topo_faces" : topo_faces}
        command_name = "PrimeMesh::TopoData/GetAdjacentTopoFacesOfTopoFaces"
        self._model._print_logs_before_command("get_adjacent_topo_faces_of_topo_faces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_adjacent_topo_faces_of_topo_faces")
        return result

    def get_adjacent_topo_edges_of_topo_edges(self, topo_edges : Iterable[int]) -> Iterable[int]:
        """ Get the adjacent topoedges for the provided topoedge ids.


        Parameters
        ----------
        topo_edges : Iterable[int]
            Ids of the topoedge.

        Returns
        -------
        Iterable[int]
            Returns the list of adjacent topoedge ids.


        Examples
        --------
        >>> topo_edges_of_topo_edges = topo_data.get_adjacent_topo_edges_of_topo_edges(topo_edges)

        """
        args = {"topo_edges" : topo_edges}
        command_name = "PrimeMesh::TopoData/GetAdjacentTopoEdgesOfTopoEdges"
        self._model._print_logs_before_command("get_adjacent_topo_edges_of_topo_edges", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_adjacent_topo_edges_of_topo_edges")
        return result

    @property
    def id(self):
        """ Get the id of TopoData."""
        return self._id

    @property
    def name(self):
        """ Get the name of TopoData."""
        return self._name



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class PrismControl(CoreObject):
    """PrismControl allows you to generate prisms.

    PrismControl allows you to control generation of prisms. Controls include setting the face scope, volume scope and growth parameters.
    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """ Initialize PrismControl """
        self._model = model
        self._comm = model._communicator
        self._id = id
        self._object_id = object_id
        self._name = name
        self._freeze()

    def set_growth_params(self, prism_control_growth_params : PrismControlGrowthParams):
        """ Set growth parameters for prism control.


        Parameters
        ----------
        prism_control_growth_params : PrismControlGrowthParams
            Parameters to set prism control growth.

        Examples
        --------
        >>> results = prism_control.set_growth_params(PrismControlGrowthParams(model=model))

        """
        args = {"prism_control_growth_params" : prism_control_growth_params._jsonify()}
        command_name = "PrimeMesh::PrismControl/SetGrowthParams"
        self._model._print_logs_before_command("set_growth_params", args)
        self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_growth_params")

    def set_surface_scope(self, entities : ScopeDefinition) -> SetScopeResults:
        """ Sets the surface control scope.


        Parameters
        ----------
        entities : ScopeDefinition
            Scope definition entities.

        Returns
        -------
        SetScopeResults
            Returns SetScopeResults.


        Examples
        --------
        >>> results = prism_control.set_surface_scope(entities)

        """
        args = {"entities" : entities._jsonify()}
        command_name = "PrimeMesh::PrismControl/SetSurfaceScope"
        self._model._print_logs_before_command("set_surface_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_surface_scope", SetScopeResults(model = self._model, json_data = result))
        return SetScopeResults(model = self._model, json_data = result)

    def set_volume_scope(self, entities : ScopeDefinition) -> SetScopeResults:
        """ Sets the volume control scope.


        Parameters
        ----------
        entities : ScopeDefinition
            Scope definition entities.

        Returns
        -------
        SetScopeResults
            Returns SetScopeResults.


        Examples
        --------
        >>> results = prism_control.set_volume_scope(entities)

        """
        args = {"entities" : entities._jsonify()}
        command_name = "PrimeMesh::PrismControl/SetVolumeScope"
        self._model._print_logs_before_command("set_volume_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("set_volume_scope", SetScopeResults(model = self._model, json_data = result))
        return SetScopeResults(model = self._model, json_data = result)

    def get_surface_scope(self) -> ScopeDefinition:
        """ Gets the surface control scope.


        Returns
        -------
        ScopeDefinition
            Returns the ScopeDefinition.


        Examples
        --------
        >>> scope = prism_control.get_surface_scope()

        """
        args = {}
        command_name = "PrimeMesh::PrismControl/GetSurfaceScope"
        self._model._print_logs_before_command("get_surface_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_surface_scope", ScopeDefinition(model = self._model, json_data = result))
        return ScopeDefinition(model = self._model, json_data = result)

    def get_volume_scope(self) -> ScopeDefinition:
        """ Gets the volume control scope.


        Returns
        -------
        ScopeDefinition
            Returns the ScopeDefinition.


        Examples
        --------
        >>> scope = prism_control.get_volume_scope()

        """
        args = {}
        command_name = "PrimeMesh::PrismControl/GetVolumeScope"
        self._model._print_logs_before_command("get_volume_scope", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_volume_scope", ScopeDefinition(model = self._model, json_data = result))
        return ScopeDefinition(model = self._model, json_data = result)

    @property
    def id(self):
        """ Get the id of PrismControl."""
        return self._id

    @property
    def name(self):
        """ Get the name of PrismControl."""
        return self._name



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class SurfaceUtilities(CoreObject):
    """Performs various general surface utilities algorithms. For example, copy zonelets, resolve surface intersections.

    """

    def __init__(self, model: CommunicationManager):
        """ Initialize SurfaceUtilities """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::SurfaceUtilities/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for SurfaceUtilities. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for SurfaceUtilities. """
        command_name = "PrimeMesh::SurfaceUtilities/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def get_bounding_box_of_zonelets(self, zonelets : Iterable[int]) -> BoundingBox:
        """ Gets bounding box of given zonelets.


        Parameters
        ----------
        zonelets : Iterable[int]
            Ids of zonelets.

        Returns
        -------
        BoundingBox
            Returns bounding of box of given zonelets.


        Examples
        --------
        >>> zonelets = part.get_face_zonelets() + part.get_edge_zonelets()
        >>> bounding_box = surface_utilities.get_bounding_box_of_zonelets(zonelets)

        """
        args = {"zonelets" : zonelets}
        command_name = "PrimeMesh::SurfaceUtilities/GetBoundingBoxOfZonelets"
        self._model._print_logs_before_command("get_bounding_box_of_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_bounding_box_of_zonelets", BoundingBox(model = self._model, json_data = result))
        return BoundingBox(model = self._model, json_data = result)

    def copy_face_zonelets(self, face_zonelets : Iterable[int], target_part_id : int, params : CopyZoneletsParams) -> CopyZoneletsResults:
        """ Copy face zonelets.


        Parameters
        ----------
        face_zonelets : Iterable[int]
            Ids of face zonelets to be copied.
        target_part_id : int
            Part id to be used to move the copied zonelets.
        params : CopyZoneletsParams
            Parameters to copy face zonelets.

        Returns
        -------
        CopyZoneletsResults
            Returns the CopyZoneletsResults.


        Examples
        --------
        >>>> surfaceutil = SurfaceUtilities(model = model)
        >>>> surfaceutil = surfaceutil.copy_face_zonelets(face_zonelets, target_part_id = new_part.id, prime.CopyZoneletsParams(model = model))

        """
        args = {"face_zonelets" : face_zonelets,
        "target_part_id" : target_part_id,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceUtilities/CopyFaceZonelets"
        self._model._print_logs_before_command("copy_face_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("copy_face_zonelets", CopyZoneletsResults(model = self._model, json_data = result))
        return CopyZoneletsResults(model = self._model, json_data = result)

    def fill_holes_at_plane(self, part_id : int, face_zonelets : Iterable[int], plane_points : Iterable[float], params : FillHolesAtPlaneParams) -> FillHolesAtPlaneResults:
        """ Fill holes in given face zonelets at given plane.


        Parameters
        ----------
        part_id : int
            Id of part to associate face zonelets created to fill hole.
        face_zonelets : Iterable[int]
            Ids of face zonelets to be used to find holes.
        plane_points : Iterable[float]
            Coordinates of three points to define plane.
        params : FillHolesAtPlaneParams
            Parameters to fill holes.

        Returns
        -------
        FillHolesAtPlaneResults
            Return the FillHolesAtPlaneResults.


        Examples
        --------
        >>> params = prime.FillHolesAtPlaneParams(model = model)
        >>> results = surface_utils.fill_holes_at_plane(part_id, face_zonelets, plane_points, params)

        """
        args = {"part_id" : part_id,
        "face_zonelets" : face_zonelets,
        "plane_points" : plane_points,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceUtilities/FillHolesAtPlane"
        self._model._print_logs_before_command("fill_holes_at_plane", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("fill_holes_at_plane", FillHolesAtPlaneResults(model = self._model, json_data = result))
        return FillHolesAtPlaneResults(model = self._model, json_data = result)

    def create_cap_on_face_zonelets(self, part_id : int, face_zonelets : Iterable[int], params : CreateCapParams) -> CreateCapResults:
        """ Create caps on the given face zonelets.


        Parameters
        ----------
        part_id : int
            Id of part to associate face zonelets created to cap.
        face_zonelets : Iterable[int]
            Ids of face zonelets to be used to find caps.
        params : CreateCapParams
            Parameters to create caps.

        Returns
        -------
        CreateCapResults
            Returns the CreateCapResults.


        Examples
        --------
        >>> params = prime.CreateCapParams(model = model)
        >>> results = surface_utils.cap_face_zonelets(part_id, face_zonelets, params)

        """
        args = {"part_id" : part_id,
        "face_zonelets" : face_zonelets,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceUtilities/CreateCapOnFaceZonelets"
        self._model._print_logs_before_command("create_cap_on_face_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_cap_on_face_zonelets", CreateCapResults(model = self._model, json_data = result))
        return CreateCapResults(model = self._model, json_data = result)

    def delete_unwetted_surfaces(self, face_zonelet_ids : Iterable[int], live_material_point_names : List[str], params : DeleteUnwettedParams) -> DeleteUnwettedResult:
        """ Delete unwetted surfaces based on material point list.


        Parameters
        ----------
        face_zonelet_ids : Iterable[int]
            Ids of face zonelets.
        live_material_point_names : List[str]
            Names of material points.
        params : DeleteUnwettedParams
            DeleteUnwettedParams to define material points.

        Returns
        -------
        DeleteUnwettedResult
            Return a DeleteUnwettedResult.


        Examples
        --------
        >>> result = surf_utils.delete_unwetted_surfaces(zonelets, live_mpt_names, prime.DeleteUnwettedParams(model))

        """
        args = {"face_zonelet_ids" : face_zonelet_ids,
        "live_material_point_names" : live_material_point_names,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceUtilities/DeleteUnwettedSurfaces"
        self._model._print_logs_before_command("delete_unwetted_surfaces", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("delete_unwetted_surfaces", DeleteUnwettedResult(model = self._model, json_data = result))
        return DeleteUnwettedResult(model = self._model, json_data = result)

    def resolve_intersections(self, face_zonelet_ids : Iterable[int], params : ResolveIntersectionsParams) -> ResolveIntersectionResult:
        """ Resolve facezonelets intersections.


        Parameters
        ----------
        face_zonelet_ids : Iterable[int]
            Ids of face zonelets.
        params : ResolveIntersectionsParams
            ResolveIntersectionsParams for resolve intersection.

        Returns
        -------
        ResolveIntersectionResult
            Return a ResolveIntersectionResult.


        Examples
        --------
        >>> result = surf_utils.resolve_intersections(zonelets, prime.ResolveIntersectionsParams(model))

        """
        args = {"face_zonelet_ids" : face_zonelet_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceUtilities/ResolveIntersections"
        self._model._print_logs_before_command("resolve_intersections", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("resolve_intersections", ResolveIntersectionResult(model = self._model, json_data = result))
        return ResolveIntersectionResult(model = self._model, json_data = result)

    def smooth_dihedral_face_nodes(self, zonelets : Iterable[int], params : SmoothDihedralFaceNodesParams) -> SmoothDihedralFaceNodesResults:
        """ Performs a smoothing operation to eliminate sharp corners at locations where the input face zonelets intersect.


        Parameters
        ----------
        zonelets : Iterable[int]
            List of input face zonelet ids.
        params : SmoothDihedralFaceNodesParams
            Parameters to control the smoothing operation.

        Returns
        -------
        SmoothDihedralFaceNodesResults
            Return a SmoothDihedralFaceNodesResults.


        Examples
        --------
        >>> result = surf_utils.smooth_dihedral_face_nodes(zonelets, params)

        """
        args = {"zonelets" : zonelets,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceUtilities/SmoothDihedralFaceNodes"
        self._model._print_logs_before_command("smooth_dihedral_face_nodes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("smooth_dihedral_face_nodes", SmoothDihedralFaceNodesResults(model = self._model, json_data = result))
        return SmoothDihedralFaceNodesResults(model = self._model, json_data = result)

    def refine_at_contacts(self, part_ids : Iterable[int], params : RefineAtContactsParams) -> RefineAtContactsResults:
        """ Refine face elements in contact with other parts.


        Parameters
        ----------
        part_ids : Iterable[int]
            Input part ids.
        params : RefineAtContactsParams
            Parameters to refine at contacts.

        Returns
        -------
        RefineAtContactsResults
            Returns the RefineAtContactsResults.


        Examples
        --------
        >>> params = prime.RefineAtContactsParams(model = model)
        >>> result = surf_utils.refine_at_contacts(part_ids, params)

        """
        args = {"part_ids" : part_ids,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceUtilities/RefineAtContacts"
        self._model._print_logs_before_command("refine_at_contacts", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("refine_at_contacts", RefineAtContactsResults(model = self._model, json_data = result))
        return RefineAtContactsResults(model = self._model, json_data = result)

    def add_thickness(self, zonelets : Iterable[int], params : AddThicknessParams) -> AddThicknessResults:
        """ Adds thickness to the selected list of face zonelet ids.


        Parameters
        ----------
        zonelets : Iterable[int]
            List of input face zonelet ids.
        params : AddThicknessParams
            Parameters to control the add thickness operation.

        Returns
        -------
        AddThicknessResults
            Returns the AddThicknessResults.


        Examples
        --------
        >>> result = surf_utils.add_thickness(zonelets, params)

        """
        args = {"zonelets" : zonelets,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceUtilities/AddThickness"
        self._model._print_logs_before_command("add_thickness", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("add_thickness", AddThicknessResults(model = self._model, json_data = result))
        return AddThicknessResults(model = self._model, json_data = result)

    def create_boi(self, scope : ScopeDefinition, params : CreateBOIParams) -> CreateBOIResults:
        """ Creates BOI to the selected list of face zonelet ids.


        Parameters
        ----------
        scope : ScopeDefinition
            Scope of zonelets.
        params : CreateBOIParams
            Parameters to control the BOI creation operation.

        Returns
        -------
        CreateBOIResults
            Returns the BOIResults.


        Examples
        --------
        >>> result = surf_utils.create_surface_boi(zonelets, params)

        """
        args = {"scope" : scope._jsonify(),
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceUtilities/CreateBOI"
        self._model._print_logs_before_command("create_boi", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_boi", CreateBOIResults(model = self._model, json_data = result))
        return CreateBOIResults(model = self._model, json_data = result)

    def create_contact_patch(self, source_scope : ScopeDefinition, target_scope : ScopeDefinition, params : CreateContactPatchParams) -> CreateContactPatchResults:
        """ Creates contact patches.


        Parameters
        ----------
        source_scope : ScopeDefinition
            Scope of source zonelets.
        target_scope : ScopeDefinition
            Scope of target zonelets which is to be offsetted for contact patch creation.
        params : CreateContactPatchParams
            Parameters to control the contact patch creation operation.

        Returns
        -------
        CreateContactPatchResults
            Returns the CreateContactPatchResults.


        Examples
        --------
        >>> result = surf_utils.create_contact_patch(zonelets, params)

        """
        args = {"source_scope" : source_scope._jsonify(),
        "target_scope" : target_scope._jsonify(),
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceUtilities/CreateContactPatch"
        self._model._print_logs_before_command("create_contact_patch", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("create_contact_patch", CreateContactPatchResults(model = self._model, json_data = result))
        return CreateContactPatchResults(model = self._model, json_data = result)



""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class CollapseTool(CoreObject):
    """Performs various collapse operations. For example, split and collapse on face zonelets.

    """

    def __init__(self, model: CommunicationManager):
        """ Initialize CollapseTool """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::CollapseTool/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for CollapseTool. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for CollapseTool. """
        command_name = "PrimeMesh::CollapseTool/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def split_and_collapse_on_zonelets(self, part_id : int, face_zonelets : Iterable[int], register_id : int, split_params : SplitParams, collapse_params : CollapseParams) -> CollapseResults:
        """ Split and collapse elements on face zonelets with the specified register id.


        Parameters
        ----------
        part_id : int
            Id of a part.
        face_zonelets : Iterable[int]
            Ids of face zonelets.
        register_id : int
            Id used to identify face elements to split and collapse.
        split_params : SplitParams
            Parameters to split longest edge of face elements before collapse.
        collapse_params : CollapseParams
            Parameters to collapse shortest edge of face elements.

        Returns
        -------
        CollapseResults
            Returns the CollapseResults.

        Examples
        --------
        collapse_tool = prime.CollapseTool(model = model)
        results = collapse_tool.split_and_collapse_on_zonelets(part.id,
        part.get_face_zonelets(),
        register_id,
        prime.SplitParams(model=model),
        prime.CollapseParams(model=model))

        """
        args = {"part_id" : part_id,
        "face_zonelets" : face_zonelets,
        "register_id" : register_id,
        "split_params" : split_params._jsonify(),
        "collapse_params" : collapse_params._jsonify()}
        command_name = "PrimeMesh::CollapseTool/SplitAndCollapseOnZonelets"
        self._model._print_logs_before_command("split_and_collapse_on_zonelets", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("split_and_collapse_on_zonelets", CollapseResults(model = self._model, json_data = result))
        return CollapseResults(model = self._model, json_data = result)



"""Examples module for PyPrimeMesh
"""
import os
from typing import Optional, Union

from .download import download_file

__all__ = [
    'clear_download_cache',
    'download_file',
    'download_elbow_pmdat',
    'download_elbow_fmd',
    'download_elbow_scdoc',
    'download_bracket_fmd',
    'download_bracket_scdoc',
    'download_toy_car_pmdat',
    'download_toy_car_fmd',
    'download_toy_car_scdoc',
    'download_pipe_tee_pmdat',
    'download_pipe_tee_fmd',
]

_DOWNLOADS = []


def download_elbow_pmdat(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download PMDAT file for the mixing elbow example

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used
    force: bool
        If true, the file is always downloaded.
        If false, an existing file in the cache may be re-used.

    Returns
    -------
    str
        Local path to the downloaded file

    Raises
    ------
    ValueError
        When the provided destination path does not exist on file

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     elbow = prime_examples.download_elbow_pmdat()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.read_pmdat(elbow, params=prime.FileReadParams(model))
    >>>     print(model)

    """
    if destination is not None and not os.path.isdir(destination):
        raise ValueError('destination directory provided does not exist')
    file = download_file(
        'mixing_elbow.pmdat', 'pyprimemesh', 'mixing_elbow', destination=destination, force=force
    )
    _DOWNLOADS.append(file)
    return file


def download_elbow_fmd(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download FMD file for the mixing elbow example

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used
    force: bool
        If true, the file is always downloaded.
        If false, an existing file in the cache may be re-used.

    Returns
    -------
    str
        Local path to the downloaded file

    Raises
    ------
    ValueError
        When the provided destination path does not exist on file

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     elbow = prime_examples.download_elbow_fmd()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(elbow, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    if destination is not None and not os.path.isdir(destination):
        raise ValueError('destination directory provided does not exist')
    file = download_file(
        'mixing_elbow.fmd', 'pyprimemesh', 'mixing_elbow', destination=destination, force=force
    )
    _DOWNLOADS.append(file)
    return file


def download_elbow_scdoc(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download SCDOC file for the mixing elbow example

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used
    force: bool
        If true, the file is always downloaded.
        If false, an existing file in the cache may be re-used.

    Returns
    -------
    str
        Local path to the downloaded file

    Raises
    ------
    ValueError
        When the provided destination path does not exist on file

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     elbow = prime_examples.download_elbow_scdoc()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(elbow, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    if destination is not None and not os.path.isdir(destination):
        raise ValueError('destination directory provided does not exist')
    file = download_file(
        'mixing_elbow.scdoc', 'pyprimemesh', 'mixing_elbow', destination=destination, force=force
    )
    _DOWNLOADS.append(file)
    return file


def download_bracket_fmd(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download FMD file for the bracket scaffold example

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used
    force: bool
        If true, the file is always downloaded.
        If false, an existing file in the cache may be re-used.

    Returns
    -------
    str
        Local path to the downloaded file

    Raises
    ------
    ValueError
        When the provided destination path does not exist on file

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     bracket = prime_examples.download_bracket_fmd()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(bracket, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    if destination is not None and not os.path.isdir(destination):
        raise ValueError('destination directory provided does not exist')
    file = download_file(
        'bracket_mid_surface.fmd',
        'pyprimemesh',
        'bracket_scaffold',
        destination=destination,
        force=force,
    )
    _DOWNLOADS.append(file)
    return file


def download_bracket_scdoc(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download SCDOC file for the bracket scaffold example

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used
    force: bool
        If true, the file is always downloaded.
        If false, an existing file in the cache may be re-used.

    Returns
    -------
    str
        Local path to the downloaded file

    Raises
    ------
    ValueError
        When the provided destination path does not exist on file

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     bracket = prime_examples.download_bracket_scdoc()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(bracket, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    if destination is not None and not os.path.isdir(destination):
        raise ValueError('destination directory provided does not exist')
    file = download_file(
        'bracket_mid_surface.scdoc',
        'pyprimemesh',
        'bracket_scaffold',
        destination=destination,
        force=force,
    )
    _DOWNLOADS.append(file)
    return file


def download_toy_car_pmdat(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download PMDAT file for the toy car example

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used
    force: bool
        If true, the file is always downloaded.
        If false, an existing file in the cache may be re-used.

    Returns
    -------
    str
        Local path to the downloaded file

    Raises
    ------
    ValueError
        When the provided destination path does not exist on file

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     toy_car = prime_examples.download_toy_car_pmdat()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.read_pmdat(toy_car, params=prime.FileReadParams(model))
    >>>     print(model)

    """
    if destination is not None and not os.path.isdir(destination):
        raise ValueError('destination directory provided does not exist')
    file = download_file(
        'toy_car.pmdat', 'pyprimemesh', 'toy_car', destination=destination, force=force
    )
    _DOWNLOADS.append(file)
    return file


def download_toy_car_fmd(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download FMD file for the toy car example

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used
    force: bool
        If true, the file is always downloaded.
        If false, an existing file in the cache may be re-used.

    Returns
    -------
    str
        Local path to the downloaded file

    Raises
    ------
    ValueError
        When the provided destination path does not exist on file

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     toy_car = prime_examples.download_toy_car_fmd()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(toy_car, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    if destination is not None and not os.path.isdir(destination):
        raise ValueError('destination directory provided does not exist')
    file = download_file(
        'toy_car.fmd', 'pyprimemesh', 'toy_car', destination=destination, force=force
    )
    _DOWNLOADS.append(file)
    return file


def download_toy_car_scdoc(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download SCDOC file for the toy car example

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used
    force: bool
        If true, the file is always downloaded.
        If false, an existing file in the cache may be re-used.

    Returns
    -------
    str
        Local path to the downloaded file

    Raises
    ------
    ValueError
        When the provided destination path does not exist on file

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     toy_car = prime_examples.download_toy_car_scdoc()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(toy_car, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    if destination is not None and not os.path.isdir(destination):
        raise ValueError('destination directory provided does not exist')
    file = download_file(
        'toy_car.scdoc', 'pyprimemesh', 'toy_car', destination=destination, force=force
    )
    _DOWNLOADS.append(file)
    return file


def download_pipe_tee_pmdat(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download PMDAT file for the pipe tee example

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used
    force: bool
        If true, the file is always downloaded.
        If false, an existing file in the cache may be re-used.

    Returns
    -------
    str
        Local path to the downloaded file

    Raises
    ------
    ValueError
        When the provided destination path does not exist on file

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     pipe_tee = prime_examples.download_pipe_tee_pmdat()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.read_pmdat(pipe_tee, params=prime.FileReadParams(model))
    >>>     print(model)

    """
    if destination is not None and not os.path.isdir(destination):
        raise ValueError('destination directory provided does not exist')
    file = download_file(
        'pipe_tee.pmdat', 'pyprimemesh', 'pipe_tee', destination=destination, force=force
    )
    _DOWNLOADS.append(file)
    return file


def download_pipe_tee_fmd(
    destination: Optional[str] = None, force: bool = False
) -> Union[str, os.PathLike]:
    """Download FMD file for the pipe tee example

    Parameters
    ----------
    destination: Optional[str]
        Destination for the file to be downloaded.
        If nothing is provided, the default path in app data is used
    force: bool
        If true, the file is always downloaded.
        If false, an existing file in the cache may be re-used.

    Returns
    -------
    str
        Local path to the downloaded file

    Raises
    ------
    ValueError
        When the provided destination path does not exist on file

    Examples
    --------
    >>> import ansys.meshing.prime as prime
    >>> import ansys.meshing.prime.examples as prime_examples
    >>> with prime.launch_prime() as session:
    >>>     model = session.model
    >>>     pipe_tee = prime_examples.download_pipe_tee_fmd()
    >>>     with prime.FileIO(model) as io:
    >>>         _ = io.import_cad(pipe_tee, params=prime.ImportCADParams(model))
    >>>     print(model)

    """
    if destination is not None and not os.path.isdir(destination):
        raise ValueError('destination directory provided does not exist')
    file = download_file(
        'pipe_tee.fmd', 'pyprimemesh', 'pipe_tee', destination=destination, force=force
    )
    _DOWNLOADS.append(file)
    return file


def clear_download_cache():
    """Clears the cache of downloaded files"""
    [os.remove(_file) for _file in _DOWNLOADS]
    _DOWNLOADS.clear()
